@@ -10,9 +10,11 @@
 # autoconf   - configure
 #
 
+rm -rf autom4te.cache
 touch AUTHORS ChangeLog README
-autoheader
-aclocal
-libtoolize --copy --force
-automake --add-missing --copy
-autoconf
+autoreconf --verbose --install --force
+#autoheader
+#aclocal
+#libtoolize --copy --force
+#automake --add-missing --copy
+#autoconf
