@@ -55,9 +55,8 @@ Pittsburgh, PA.
 /* This string is used in RPC initialization calls to ensure that the
 runtime system and the header files are mutually consistent.  Also
 passed across on RPC2_NewBinding for advisory information to other
-side.  Changes to this string may cause RPC2_OLDVERSION to be returned
-on RPC2_NewBinding()s.  For really minor changes alter rpc2_LastEdit
-in globals.c.  */
+side. Changes to this string may cause RPC2_OLDVERSION to be returned
+on RPC2_NewBinding()s. */
 /* #define RPC2_VERSION "Version 14.0: Satya, 6 May 1988, 10:00" */
 #define RPC2_VERSION "Version 15.0: JH, 10 Dec 1998, 12:00"
 
