@@ -93,7 +93,7 @@ typedef union YYSTYPE
     int         u_int;
 }
 /* Line 1489 of yacc.c.  */
-#line 97 "y.tab.h"
+#line 97 "rp2gen_parser.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
