@@ -222,17 +222,15 @@ struct CEntry *rpc2_AllocConn(struct RPC2_addrinfo *addr)
     ce->PrivatePtr = NULL;
     ce->SideEffectPtr = NULL;
     ce->Color = 0;
-    ce->RTT = 0;
-    ce->RTTVar = 0;
-    ce->LowerLimit= LOWERLIMIT;  /* usec */
-    ce->Retry_N = Retry_N;
-    ce->Retry_Beta = (struct timeval *)malloc(sizeof(struct timeval)*(2+Retry_N));
-    memcpy(ce->Retry_Beta, Retry_Beta, sizeof(struct timeval)*(2+Retry_N));
-    ce->SaveResponse = SaveResponse;  /* structure assignment */
+
+    ce->KeepAlive = KeepAlive;
+    ce->SaveResponse.tv_usec = (2 * KeepAlive.tv_usec) % 1000000;
+    ce->SaveResponse.tv_sec = (2 * KeepAlive.tv_usec) / 1000000;
+    ce->SaveResponse.tv_sec += 2 * KeepAlive.tv_sec;
+
     ce->MySl = NULL;
     ce->HeldPacket = NULL;
     ce->reqsize = 0;
-    ce->respsize = 0;
     ce->HostInfo = rpc2_GetHost(addr);
     assert(ce->HostInfo);
     ce->Filter.FromWhom = ANY;
@@ -261,12 +259,11 @@ void rpc2_FreeConn(RPC2_Handle whichConn)
     assert(ce && ce->MagicNumber == OBJ_CENTRY);
     rpc2_FreeConns++;
 
-    free(ce->Retry_Beta);
     if (ce->HeldPacket != NULL)
 	RPC2_FreeBuffer(&ce->HeldPacket);
     if (ce->MySl != NULL) {
 	rpc2_DeactivateSle(ce->MySl);
-    	rpc2_FreeSle(&ce->MySl);
+	rpc2_FreeSle(&ce->MySl);
     }
 
     /* Scan the hold queue and purge the request for this connection */
@@ -285,8 +282,8 @@ void rpc2_FreeConn(RPC2_Handle whichConn)
     SetRole(ce, FREE);
 
     /* clear encryption state */
-    secure_setup_encrypt(&ce->sa, NULL, NULL, NULL, 0);
-    secure_setup_decrypt(&ce->sa, NULL, NULL, NULL, 0);
+    secure_setup_encrypt(0, &ce->sa, NULL, NULL, NULL, 0);
+    secure_setup_decrypt(0, &ce->sa, NULL, NULL, NULL, 0);
 
     /* move the conn entry over to the freelist */
     list_del(&ce->connlist);
@@ -317,7 +314,7 @@ void rpc2_ReapDeadConns(void)
 	if (!ce->PrivatePtr && TestRole(ce, SERVER) &&
 	    ce->LastRef + RPC2_DEAD_CONN_TIMEOUT < now)
 	{
-	    say(0, RPC2_DebugLevel, "Reaping dead connection %#x\n",
+	    say(1, RPC2_DebugLevel, "Reaping dead connection %#x\n",
 		ce->UniqueCID);
 	    RPC2_Unbind(ce->UniqueCID);
 	}
@@ -445,7 +442,7 @@ rpc2_ConnFromBindInfo(struct RPC2_addrinfo *addr,
 			rbn->Unique == whichUnique &&
 			RPC2_cmpaddrinfo(rbn->addr, addr))
 		    {
-			say(0, RPC2_DebugLevel, "RBCache hit after %d tries\n", i+1);
+			say(1, RPC2_DebugLevel, "RBCache hit after %d tries\n", i+1);
 			ce = rpc2_GetConn(rbn->MyConn);
 			/* can't test the state because OPENKIMONO connections
 			 * will already be in S_AWAITREQUEST state */
@@ -460,7 +457,7 @@ rpc2_ConnFromBindInfo(struct RPC2_addrinfo *addr,
 		    else next--;
 	    }
 	    
-	    say(0, RPC2_DebugLevel, "RBCache miss after %d tries\n", RBSIZE);
+	    say(1, RPC2_DebugLevel, "RBCache miss after %d tries\n", RBSIZE);
     }
     
     /* It was not in the RBCache; scan all the connections */
@@ -482,14 +479,14 @@ rpc2_ConnFromBindInfo(struct RPC2_addrinfo *addr,
 	     TestState(ce, SERVER, S_AWAITINIT3)) &&
 	    RPC2_cmpaddrinfo(ce->HostInfo->Addr, addr))
 	{
-	    say(0, RPC2_DebugLevel,
+	    say(1, RPC2_DebugLevel,
 		"Match after searching %d connection entries\n", j);
 	    /* and put the CE at the head of it's hashbucket */
 	    __rehash_ce(ce);
 	    return(ce);
 	}
     }
-    say(0, RPC2_DebugLevel, "No match after searching %ld connections\n",
+    say(1, RPC2_DebugLevel, "No match after searching %ld connections\n",
         rpc2_ConnCount);
 
     return(NULL);
