@@ -192,14 +192,8 @@ EXIT_MRPC(long code, int HowMany, RPC2_Integer *RCList, MultiCon *mcon)
     /* copy arguments into the return code lists, possibly translating 
        error codes */
     if (RCList) {
-	for (i = 0; i < HowMany; i++ ) {
-	    RPC2_Integer rc;
-	    rc = mcon[i].retcode;
-#ifdef ERRORTR
-	    rc = RPC2_R2SError(rc);
-#endif 
-	    RCList[i] = rc;
-	}
+	for (i = 0; i < HowMany; i++ )
+	    RCList[i] = mcon[i].retcode;
     }
 
     FreeMultiCon(HowMany, mcon);
@@ -511,7 +505,7 @@ static long mrpc_SendPacketsReliably(
 	/* XXX we should have the size of the expected reply packet */
 	rc = rpc2_RetryInterval(mcon[thispacket].ceaddr, 0, &slp->RInterval,
 				mcon[thispacket].req->Prefix.LengthOfPacket,
-				sizeof(struct RPC2_PacketHeader));
+				sizeof(struct RPC2_PacketHeader), 0);
 	assert(rc == 0);
 	rpc2_ActivateSle(slp, &slp->RInterval);
 
@@ -637,7 +631,7 @@ static long mrpc_SendPacketsReliably(
 		    rc = rpc2_RetryInterval(c_entry, slp->RetryIndex,
 					    &slp->RInterval,
 				    mcon[thispacket].req->Prefix.LengthOfPacket,
-					    sizeof(struct RPC2_PacketHeader));
+					   sizeof(struct RPC2_PacketHeader), 0);
 
 		    if (rc) {
 			say(9, RPC2_DebugLevel, "Request failed on 0x%p\n", c_entry);
