@@ -91,7 +91,7 @@ long RPC2_Init(char *VId,		/* magic version string */
     rpc2_tracefile = stderr;
 
     rpc2_Enter();
-    say(0, RPC2_DebugLevel, "RPC2_Init()\n");
+    say(1, RPC2_DebugLevel, "RPC2_Init()\n");
     say(999, RPC2_DebugLevel, "Runtime system version: \"%s\"\n", RPC2_VERSION);
 
     if (strcmp(VId, RPC2_VERSION) != 0)
@@ -112,15 +112,10 @@ long RPC2_Init(char *VId,		/* magic version string */
     if (RPC2_Preferred_Keysize > 64)
 	RPC2_Preferred_Keysize /= 8;
 
-    /* Do we accept only secure connections, default is no. This can
-     * be enabled by setting the RPC2SEC_ONLY environment variable.
-     * (and forced as disabled by setting RPC2SEC_ONLY=0/false/no)
-     *
-     * At some point once every client and server is 'expected' to run
-     * rpc2-2.0 or later we can change the default behaviour to accept
-     * only secure connections. */
+    /* Do we accept only secure connections, default is yes. This can be
+     * disabled by setting the RPC2SEC_ONLY to 0, false, no, (nada, forgetit) */
     env = getenv("RPC2SEC_ONLY");
-    RPC2_secure_only = env && memchr("0fFnN", *env, 5) == NULL;
+    RPC2_secure_only = !env || (env && memchr("0fFnN", *env, 5) == NULL);
 
     verbose = (Options && (Options->Flags & RPC2_OPTION_VERBOSE_INIT));
     secure_init(verbose);
@@ -185,6 +180,8 @@ long RPC2_Init(char *VId,		/* magic version string */
     c = "ClockTick";
     LWP_CreateProcess(rpc2_ClockTick, 16384, LWP_NORMAL_PRIORITY, NULL, c, &ctpid);
 
+    LUA_init();
+
     if (rc != RPC2_SUCCESS)
 	say(-1, RPC2_DebugLevel, "RPC2_Init(): Exiting with error\n");
 
@@ -225,7 +222,7 @@ long RPC2_Export(IN Subsys)
     struct SubsysEntry *sp;
 
     rpc2_Enter();
-    say(0, RPC2_DebugLevel, "RPC2_Export()\n");
+    say(1, RPC2_DebugLevel, "RPC2_Export()\n");
 
     switch(Subsys->Tag)
 	{
@@ -234,7 +231,7 @@ long RPC2_Export(IN Subsys)
 	    break;
 	
 	case RPC2_SUBSYSBYNAME:
-		say(0, RPC2_DebugLevel, "RPC2_Export: obsolete SUBSYSBYNAME used!\n");
+		say(1, RPC2_DebugLevel, "RPC2_Export: obsolete SUBSYSBYNAME used!\n");
 		assert(0);
 	    break;
 
@@ -259,7 +256,7 @@ long RPC2_DeExport(IN Subsys)
     struct SubsysEntry *sp;
 
     rpc2_Enter();
-    say(0, RPC2_DebugLevel, "RPC2_DeExport()\n");
+    say(1, RPC2_DebugLevel, "RPC2_DeExport()\n");
 
     if (Subsys == NULL)
 	{/* Terminate all subsystems */
@@ -276,7 +273,7 @@ long RPC2_DeExport(IN Subsys)
 	    break;
 	
 	case RPC2_SUBSYSBYNAME:
-		say(0, RPC2_DebugLevel, "RPC2_Export: obsolete SUBSYSBYNAME used!\n");
+		say(1, RPC2_DebugLevel, "RPC2_Export: obsolete SUBSYSBYNAME used!\n");
 		assert(0);
 
 	    break;
@@ -637,7 +634,7 @@ long RPC2_Enable(RPC2_Handle whichConn)
 {
 	struct CEntry *ceaddr;
 
-	say(0, RPC2_DebugLevel, "RPC2_Enable()\n");
+	say(1, RPC2_DebugLevel, "RPC2_Enable()\n");
 
 	rpc2_Enter();
 	ceaddr = rpc2_GetConn(whichConn);
@@ -655,7 +652,7 @@ long RPC2_SetColor(Conn, Color)
     {
     struct CEntry *ceaddr;
 
-    say(0, RPC2_DebugLevel, "RPC2_SetColor()\n");
+    say(1, RPC2_DebugLevel, "RPC2_SetColor()\n");
 
     rpc2_Enter();
     ceaddr = rpc2_GetConn(Conn);
@@ -670,7 +667,7 @@ long RPC2_GetColor(Conn, Color)
     {
     struct CEntry *ceaddr;
 
-    say(0, RPC2_DebugLevel, "RPC2_GetColor()\n");
+    say(1, RPC2_DebugLevel, "RPC2_GetColor()\n");
 
     rpc2_Enter();
     ceaddr = rpc2_GetConn(Conn);
@@ -731,7 +728,7 @@ long RPC2_GetNetInfo(IN Conn, INOUT RPCLog, INOUT SELog)
     {
     struct CEntry *ceaddr;
 
-    say(0, RPC2_DebugLevel, "RPC2_GetNetInfo()\n");
+    say(1, RPC2_DebugLevel, "RPC2_GetNetInfo()\n");
 
     rpc2_Enter();
 
@@ -782,7 +779,7 @@ long RPC2_PutNetInfo(IN Conn, INOUT RPCLog, INOUT SELog)
     struct CEntry *ceaddr;
     int i;
 
-    say(0, RPC2_DebugLevel, "RPC2_PutNetInfo()\n");
+    say(1, RPC2_DebugLevel, "RPC2_PutNetInfo()\n");
 
     rpc2_Enter();
 
@@ -977,23 +974,10 @@ unsigned int rpc2_MakeTimeStamp()
 }
 
 
-void rpc2_ResetObs(obsp, ceaddr) 
-    long *obsp;
-    struct CEntry *ceaddr;
-    {
-    long delta = (ceaddr->reqsize + ceaddr->respsize) * 8 * 100 / rpc2_Bandwidth;
-    say(4, RPC2_DebugLevel, "rpc2_ResetObs: conn %#x, obs %ld, delta %ld, new %ld\n", 
-			     ceaddr->UniqueCID, *obsp, delta, *obsp-delta);
-    if (*obsp > delta)  *obsp -= delta;
-    }
-
-
 /* Retransmission timer stuff */
 void rpc2_UpdateRTT(RPC2_PacketBuffer *pb, struct CEntry *ceaddr)
 {
-    int diff; 
-    unsigned int obs, upperlimit;
-    struct timeval *beta0;
+    unsigned int obs;
     RPC2_NetLogEntry entry;
 
     if (!pb->Header.TimeStamp) return;
@@ -1001,7 +985,7 @@ void rpc2_UpdateRTT(RPC2_PacketBuffer *pb, struct CEntry *ceaddr)
     TVTOTS(&pb->Prefix.RecvStamp, obs);
     say(15, RPC2_DebugLevel, "updatertt %u %u\n", obs, pb->Header.TimeStamp);
     obs = TSDELTA(obs, pb->Header.TimeStamp);
-    RPC2_UpdateEstimates(ceaddr->HostInfo, obs, ceaddr->respsize, ceaddr->reqsize);
+    RPC2_UpdateEstimates(ceaddr->HostInfo, obs, pb->Prefix.LengthOfPacket, ceaddr->reqsize);
 
     /* 
      * Requests can be sent and received in the same tick.  
@@ -1015,49 +999,9 @@ void rpc2_UpdateRTT(RPC2_PacketBuffer *pb, struct CEntry *ceaddr)
 
     /* log the round-trip time observation in the host log */
     entry.Tag = RPC2_MEASURED_NLE;
-    entry.Value.Measured.Bytes = ceaddr->reqsize + ceaddr->respsize; //-2*sizeof(struct RPC2_PacketHeader);
+    entry.Value.Measured.Bytes = ceaddr->reqsize + pb->Prefix.LengthOfPacket;
     entry.Value.Measured.ElapsedTime = obs;
     entry.Value.Measured.Conn = ceaddr->UniqueCID;
     (void) rpc2_AppendHostLog(ceaddr->HostInfo, &entry, RPC2_MEASUREMENT);
-
-    /* smooth observation if we have a bandwidth estimate */
-    if (rpc2_Bandwidth) rpc2_ResetObs(&obs, ceaddr);
-
-    if (ceaddr->RTT == 0)
-        {
-	/* initialize estimates */
-	ceaddr->RTT = obs << RPC2_RTT_SHIFT;
-	ceaddr->RTTVar = obs << (RPC2_RTTVAR_SHIFT-1);
-        }
-    else 
-        {
-	diff = (long)obs - 1 - (ceaddr->RTT >> RPC2_RTT_SHIFT);
-	if ((ceaddr->RTT += diff) <= 0)
-	    ceaddr->RTT = 1;
-
-	if (diff < 0) diff = -diff;
-	diff -= ceaddr->RTTVar >> RPC2_RTTVAR_SHIFT;
-	if ((ceaddr->RTTVar += diff) <= 0)
-	    ceaddr->RTTVar = 1;
-        }
-
-    /* 
-     * reset the lower limit on retry interval, in microseconds for
-     * rpc2_SetRetry. It should be at least LOWERLIMIT, but no more than
-     * Retry_Beta[0]. Try RTT + (RPC2_RTTVAR_SCALE * RTTVar) first.
-     */
-    ceaddr->LowerLimit = ((ceaddr->RTT >> RPC2_RTT_SHIFT) + ceaddr->RTTVar) * 1000;
-    beta0 = &ceaddr->Retry_Beta[0];
-    upperlimit = beta0->tv_usec + beta0->tv_sec * 1000000;
-
-    if (ceaddr->LowerLimit < LOWERLIMIT) ceaddr->LowerLimit = LOWERLIMIT;
-    else if (ceaddr->LowerLimit > upperlimit) ceaddr->LowerLimit = upperlimit;
-
-    say(4, RPC2_DebugLevel, "rpc2_UpdateRTT: conn %#x, obs %d, RTT %ld, RTTVar %ld LL %lu usec\n", 
-			     ceaddr->UniqueCID, obs, ceaddr->RTT, 
-			     ceaddr->RTTVar, ceaddr->LowerLimit);
-
-    /* now adjust retransmission intervals with new Lowerlimit */
-    (void) rpc2_SetRetry(ceaddr);
 }
 
